'-----------------------------------------------------------------------------
'
' SerLCD.BAS
'
' "Matrix Orbital LCD"-Emulator for Mitsubishis M50530 compatible
' displays.
'
' Matrix Orbital have developed LCDs with serial interface. They are
' widely supported, i.e. by WinAMP. Since WinAMP doesn't support a M50530-
' compatible display directly, I was forced to construct somewhat of an
' "emulator" to bring it to life.
'
' Ing. Mario Kienspergher, OE9MKV
' Kinzi Design - Copyright (c) 2005.
' http://www.kinzi.net/
' http://www.oe9mkv.net/
'
'-----------------------------------------------------------------------------
'
' The Mitsubishi M50530 is somewhat similar but also somewhat different to
' the Hitachi HD44780 standard. I have an Optrex M50465N LCD (3x40) with the
' pinout seen below; it supports 4-bit-mode only. However, the chip seems to
' be able to use 8-bit-mode, too (according to the data sheets).
'
' The main differences are:
'
' HD44780      M50530 (Optrex DMC 50465N)     Signal
' [1]  Vss     [10] Vss                       GND
' [2]  Vdd     [11] Vdd                       +5V
' [3]  Vee     [9]  Vlcd                      see note **1
' [4]  R/S     [8]  I/OC2                     command-/data-mode
' [5]  R/W     [6]  R/W                       read-/write-mode (tie to GND)
' [6]  E       [5]  EX                        execute (strobe)
' [7]  DB0     [--]                           data bit 0 (8-bit-mode only)
' [8]  DB1     [--]                           data bit 1 (8-bit-mode only)
' [9]  DB2     [--]                           data bit 2 (8-bit-mode only)
' [10] DB3     [--]                           data bit 3 (8-bit-mode only)
' [11] DB4     [4]  DB4                       data bit 4 (4- & 8-bit-mode)
' [12] DB5     [3]  DB5                       data bit 5 (4- & 8-bit-mode)
' [13] DB6     [2]  DB6                       data bit 6 (4- & 8-bit-mode)
' [14] DB7     [1]  DB7                       data bit 7 (4- & 8-bit-mode)
'              [7]  I/OC1                     I/O-Control 1 (tie to GND)
'                                                           '
'-----------------------------------------------------------------------------
'
' Controller pinout:
'
' Portb.7...4 = DB7...DB4
' Portb.3     = <reserved>
' Portb.2     = I/OC2
' Portb.1     = I/OC1
' Portb.0     = EX
'
' Portd.6...2 = <unused>
' Portd.1     = TXD RS232
' Portd.0     = RXD RS232
'
'-----------------------------------------------------------------------------
'
' Portions of code taken from
'
' OPTREX DMC50625N LCD display controle routines for Basic Stamp 2
' (C) 2003 Hans Luyten
' Many thanks to Peter Pitzeier
'
'-----------------------------------------------------------------------------

$baud = 19200                                               ' 19200 baud serial line
$crystal = 3579545                                          ' 3.579545 MHz

Const Lcd_exe_delay = 20                                    ' standard delay is 20 s for command
Const Lcd_chr_delay = 20                                    ' standard delay is 20 s for writing data to ram
Const Lcd_clr_delay = 1250                                  ' delay for CLR/HOME is 1.25 ms
Const Lcd_pulse_delay = 1                                   ' delay for E-pulse is 1 s

Lcd_db7 Alias Portb.7
Lcd_db6 Alias Portb.6
Lcd_db5 Alias Portb.5
Lcd_db4 Alias Portb.4
Lcd_rw Alias Portb.3                                        ' Not used at the moment - for further extensions
Lcd_ioc2 Alias Portb.2
Lcd_ioc1 Alias Portb.1
Lcd_ex Alias Portb.0

Declare Function Getserial() As Byte

Dim Thedata As Byte                                         ' Variable for LCD data/char
Dim Thechar As Byte                                         ' Variable for sending characters to LCD
Dim Thecursorx As Byte                                      ' Move cursor X (column) position
Dim Thecursory As Byte                                      ' Move cursor Y (line) position

Dim Customcharnr As Byte                                    ' Nr of the customchar [0..7]

Dim I As Byte                                               ' Loop variable

'-----------------------------------------------------------------------------

Main:

Ddrb = &B11111111                                           ' All Port.b pins are outputs
Portb = &B11111111                                          ' Enables pull-ups on PORTB when PORTB lines are inputs

Ddrd = &B00000010                                           ' All Port.d pins are inputs except TXD (Portd.1)
Portd = &B11111111                                          ' Enables pull-ups on PORTD when PORTD lines are inputs

Waitms 100                                                  ' Let voltages settle on LCD

Gosub Lcd_init
Gosub Lcd_clearhome

Do
   Thechar = Getserial()

   If Thechar = 254 Then
      Gosub Lcd_checkcommand                                ' &FE = command flag
   Else
      Select Case Thechar
         Case 0 : Thechar = 244                             '
         Case 1 : Thechar = 245                             ' Map custom characters
         Case 2 : Thechar = 246                             ' With 4 lines and 160 bytes DD RAM
         Case 3 : Thechar = 247                             ' we get 96 bytes CG RAM leading into
         Case 4 : Thechar = 248                             ' 12 custom characters
         Case 5 : Thechar = 249
         Case 6 : Thechar = 250
         Case 7 : Thechar = 251
         Case 8 : Thechar = 252
         Case 9 : Thechar = 253
         Case 10 : Thechar = 254
         Case 11 : Thechar = 255
      End Select
      Gosub Lcd_writechar                                   ' Output all other chars
   End If
Loop

End

'-----------------------------------------------------------------------------
' Functions
'-----------------------------------------------------------------------------
' GetSerial()
'-----------------------------------------------------------------------------

Function Getserial() As Byte

Local Dummy As Byte

   Do
      Dummy = Ischarwaiting()
   Loop Until Dummy > 0

   Dummy = Inkey()

   Getserial = Dummy

End Function

'-----------------------------------------------------------------------------
' Subroutines
'-----------------------------------------------------------------------------
' Initialize LCD display
'-----------------------------------------------------------------------------

Lcd_init:

Lcd_ioc1 = 0                                                ' Normally the I/OC1 line is 0

Gosub Lcd_setfunction
Gosub Lcd_clearhome
Gosub Lcd_setdisplay
Gosub Lcd_setentry

Gosub Lcd_display_copyright
Waitms 3000
Gosub Lcd_clearhome

Return

'-----------------------------------------------------------------------------
' execute LCD command by pulsing LCD-E-line
'-----------------------------------------------------------------------------

Lcd_pulseout:

Lcd_ex = 1
Waitus Lcd_pulse_delay
Lcd_ex = 0
Waitus Lcd_pulse_delay

Return

'-----------------------------------------------------------------------------
' Write LCD instruction (TheData)
'-----------------------------------------------------------------------------

Lcd_writeinstr:

Lcd_db4 = Thedata.4
Lcd_db5 = Thedata.5
Lcd_db6 = Thedata.6
Lcd_db7 = Thedata.7
Lcd_ioc2 = 0

Gosub Lcd_pulseout

Lcd_db4 = Thedata.0
Lcd_db5 = Thedata.1
Lcd_db6 = Thedata.2
Lcd_db7 = Thedata.3
Lcd_ioc2 = 0

Gosub Lcd_pulseout

Waitus Lcd_exe_delay

Return

'-----------------------------------------------------------------------------
' Write character to LCD display (TheChar)
'-----------------------------------------------------------------------------

Lcd_writechar:

Lcd_db4 = Thechar.4
Lcd_db5 = Thechar.5
Lcd_db6 = Thechar.6
Lcd_db7 = Thechar.7
Lcd_ioc2 = 1

Gosub Lcd_pulseout

Lcd_db4 = Thechar.0
Lcd_db5 = Thechar.1
Lcd_db6 = Thechar.2
Lcd_db7 = Thechar.3
Lcd_ioc2 = 1

Gosub Lcd_pulseout

Waitus Lcd_chr_delay

Return

'-----------------------------------------------------------------------------
' SF: Set Function Mode
' Nibble mode, 5x7 font, 4x40 chars, 160 bytes DD RAM, 96 bytes CG RAM
' I/OC1=0 , I/OC2(RS)=0 , R/W=0
'  _______________________________________________________________________
' |fix     |fix     |4/8bit  |8/12px  |duty             |ram reg          |
' |________|________|________|________|_________________|_________________|
' |1       |1       |4bit=0  |8px=1   |1       |0       |1       |1       |
' |        |        |8bit=1  |12px=0  |        |        |        |        |
' |________|________|________|________|________|________|________|________|
'
'-----------------------------------------------------------------------------

Lcd_setfunction:

Thedata = &B11011011

Gosub Lcd_writeinstr

Return

'-----------------------------------------------------------------------------
' SD: Set Display Mode
' Display, Cursor, Underline, Blink
'  _______________________________________________________________________
' |fix     |fix     |fix     |display |cursor  |undline |crs blnk|chr blnk|
' |________|________|________|________|_________________|_________________|
' |0       |0       |1       |0=off   |0=off   |0=off   |0=off   |0=off   |
' |        |        |        |1=on    |1=on    |1=on    |1=on    |1=on    |
' |________|________|________|________|________|________|________|________|
'
'-----------------------------------------------------------------------------

Lcd_setdisplay:

Thedata = &B00110000

Gosub Lcd_writeinstr

Return

'-----------------------------------------------------------------------------
' SE: Set Entry Mode
'  _______________________________________________________________________
' |fix     |fix     |cursor  |cursor condition |display |display condition|
' |________|________|________|__________________________|_________________|
' |0       |1       |0=increm|1=in/de |1=in/de |0=increm|1=in/de |1=in/de |
' |        |        |1=decrem|on write|on read |1=decrem|on write|on read |
' |________|________|________|________|________|________|________|________|
'
'-----------------------------------------------------------------------------

Lcd_setentry:

Thedata = &B01011000
Gosub Lcd_writeinstr

Return

'-----------------------------------------------------------------------------
' CH: Clear screen, cursor home. DELAY = 1.25 ms !!
'  _______________________________________________________________________
' |fix     |fix     |fix     |fix     |fix     |fix     |fix     |fix     |
' |________|________|________|________|________|________|_________________|
' |0       |0       |0       |0       |0       |0       |0       |1       |
' |________|________|________|________|________|________|________|________|
'
'-----------------------------------------------------------------------------

Lcd_clearhome:

Thedata = &B00000001
Gosub Lcd_writeinstr

Waitus Lcd_clr_delay

Return

'-----------------------------------------------------------------------------
' MH: Move cursor to home position.
'  _______________________________________________________________________
' |fix     |fix     |fix     |fix     |fix     |fix     |fix     |fix     |
' |________|________|________|________|________|________|_________________|
' |0       |0       |0       |0       |0       |0       |1       |1       |
' |________|________|________|________|________|________|________|________|
'
'-----------------------------------------------------------------------------

Lcd_cursorhome:

Thedata = &B00000011
Gosub Lcd_writeinstr

Return

'-----------------------------------------------------------------------------
' WC: Write cursor RAM address
' I/OC1 = 1, I/OC2 = 1, R/W = 0
'  _______________________________________________________________________
' |fix     |fix     |fix     |fix     |cursor  |cursor  |display |display |
' |________|________|________|________|________|________|_________________|
' |AD7     |AD6     |AD5     |AD4     |AD3     |AD2     |AD1     |AD0     |
' |________|________|________|________|________|________|________|________|
'
'-----------------------------------------------------------------------------

Lcd_cursorxy:

Thechar = 40 * Thecursory
Thechar = Thechar + Thecursorx

Lcd_ioc1 = 1
Gosub Lcd_writechar                                         ' Use writechar because I/OC 2 has to be 1
Lcd_ioc1 = 0

Return

'-----------------------------------------------------------------------------
' LCD_Custom_Char
' Defines custom characters
'-----------------------------------------------------------------------------

Lcd_custom_char:

Customcharnr = Getserial()                                  ' Next byte holds the number of the char to customize

If Customcharnr < 12 Then
   Customcharnr = Customcharnr * 8                          ' Compute offset
   Customcharnr = Customcharnr + 160                        ' CG RAM starts at $A0

   For I = 0 To 7
      Thechar = Customcharnr + I                            ' RAM address of character pixel matrix byte
      Lcd_ioc1 = 1
      Gosub Lcd_writechar                                   ' Use writechar because I/OC 2 has to be 1
      Lcd_ioc1 = 0
      Thechar = Getserial()                                 ' Fetch pixel matrix bytes
      Gosub Lcd_writechar
   Next

End If

Return

'-----------------------------------------------------------------------------
' LCD_Custom_Char
' Defines custom characters
'-----------------------------------------------------------------------------

Define_input_keys:

Return

'-----------------------------------------------------------------------------
' LCD_checkcommand
'
' Interpretes function after command byte (&FE) as defined by Matrix Orbitals
' http://www.matrixorbitals.com/
'-----------------------------------------------------------------------------

Lcd_checkcommand:

Thechar = Getserial()                                       ' When &FE was detected, fetch next byte (= the command)

Select Case Thechar
   Case 88 : Thedata = &B00000001
             Gosub Lcd_writeinstr                           ' CH - Clear Home
   Case 72 : Thedata = &B00000011
             Gosub Lcd_writeinstr                           ' MH - Move Home
   Case 76 : Thedata = &B00011100
             Gosub Lcd_writeinstr                           ' MA - Move Cursor Left
   Case 77 : Thedata = &B00011000
             Gosub Lcd_writeinstr                           ' MA - Move Cursor Right
   Case 71 : Thecursorx = Getserial() - 1
             Thecursory = Getserial() - 1
             Gosub Lcd_cursorxy
   Case 74 : Thedata = &B00111010                           ' Turn On Underline Cursor
             Gosub Lcd_writeinstr
   Case 75 : Thedata = &B00110000                           ' Turn Off Underline Cursor
             Gosub Lcd_writeinstr
   Case 83 : Thedata = &B00111011                           ' Turn On Block Cursor
             Gosub Lcd_writeinstr
   Case 84 : Thedata = &B00110000                           ' Turn Off Block Cursor
             Gosub Lcd_writeinstr
   Case 78 : Gosub Lcd_custom_char                          ' Define Custom Character
   Case 253 : Gosub Define_input_keys
   Case 254 : Gosub Lcd_display_copyright
   Case 255 : Gosub Lcd_display_testpattern
End Select

Return

'-----------------------------------------------------------------------------
' LCD_Display_Copyright
'-----------------------------------------------------------------------------

Lcd_display_copyright:

Gosub Lcd_clearhome

Restore Datastart1

For I = 1 To 120                                            ' Display Copyright Notice
   Read Thechar
   Gosub Lcd_writechar
Next

Gosub Lcd_cursorhome
Thedata = &B00110000                                        ' Turn off all cursors
Gosub Lcd_writeinstr
Return

'-----------------------------------------------------------------------------
' LCD_Display_Testpattern
'-----------------------------------------------------------------------------

Lcd_display_testpattern:

Gosub Lcd_clearhome

Thechar = 15                                                ' Use block character

For I = 1 To 120                                            ' Display Copyright Notice
   Gosub Lcd_writechar
Next

Gosub Lcd_cursorhome
Thedata = &B00110000                                        ' Turn off all cursors
Gosub Lcd_writeinstr
Return

'-----------------------------------------------------------------------------
' End Of Program
'-----------------------------------------------------------------------------
End

'-----------------------------------------------------------------------------
' Data
'-----------------------------------------------------------------------------

Datastart1:
'     123456789 123456789 123456789 123456789!123456789 123456789 12345678 G123456789!123456789 123456789 123456789 123456789!"
Data " SerLCD/M50530 V1.01 by OE9MKV 07/2005.   Emulates a Matrix Orbital Serial LCD  XTAL 3.579545 MHz, UART 19k2, Cursor Off"
